@@ -1,4 +1,4 @@
-# Copyright 2014, 2016 IBM Corp.
+# Copyright 2014, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -13,7 +13,6 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-#
 
 from __future__ import absolute_import
 
@@ -44,14 +43,13 @@ from pypowervm.wrappers import base_partition as pvm_bp
 from pypowervm.wrappers import logical_partition as pvm_lpar
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
-from nova.virt.powervm import driver
+from nova.virt.powervm_ext import driver
 
 from nova_powervm.tests.virt import powervm
 from nova_powervm.tests.virt.powervm import fixtures as fx
 from nova_powervm.virt.powervm import exception as p_exc
 from nova_powervm.virt.powervm import live_migration as lpm
 from nova_powervm.virt.powervm import vm
-from nova_powervm.virt.powervm import volume as vol_attach
 
 LOG = logging.getLogger(__name__)
 logging.basicConfig()
@@ -119,7 +117,8 @@ class TestPowerVMDriver(test.TestCase):
         self.build_tx_feed = self.useFixture(fixtures.MockPatch(
             'pypowervm.tasks.partition.build_active_vio_feed_task')).mock
 
-        self.stg_ftsk = pvm_tx.FeedTask('fake', pvm_vios.VIOS.getter(self.apt))
+        self.stg_ftsk = pvm_tx.FeedTask('fake',
+                                        [mock.Mock(spec=pvm_vios.VIOS)])
         self.build_tx_feed.return_value = self.stg_ftsk
 
         self.scrub_stg = self.useFixture(fixtures.MockPatch(
@@ -162,15 +161,6 @@ class TestPowerVMDriver(test.TestCase):
         self.lpm_inst.uuid = 'inst1'
         self.drv.live_migrations = {'inst1': self.lpm}
 
-    def _vol_drv_maps(self):
-        VOLUME_DRIVER_MAPPINGS = {
-            'fibre_channel': vol_attach.FC_STRATEGY_MAPPING[
-                driver.CONF.powervm.fc_attach_strategy.lower()],
-            'iscsi': vol_attach.NETWORK_STRATEGY_MAPPING[
-                driver.CONF.powervm.network_attach_strategy.lower()],
-        }
-        return VOLUME_DRIVER_MAPPINGS
-
     def test_driver_create(self):
         """Validates that a driver of the PowerVM type can be initialized."""
         test_drv = driver.PowerVMDriver(fake.FakeVirtAPI())
@@ -303,7 +293,7 @@ class TestPowerVMDriver(test.TestCase):
                 '.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -381,7 +371,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -436,7 +426,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -497,7 +487,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -551,7 +541,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.ConnectVolume')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.ConnectDisk')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.FindDisk')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs')
@@ -663,7 +653,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg.'
                 'execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -671,7 +661,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
     @mock.patch('nova_powervm.virt.powervm.tasks.vm.UpdateIBMiSettings.'
                 'execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_get_boot_connectivity_type')
     @mock.patch('pypowervm.tasks.power.power_on')
     def test_spawn_ibmi(self, mock_pwron, mock_boot_conn_type,
@@ -721,7 +711,7 @@ class TestPowerVMDriver(test.TestCase):
                 '.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -729,7 +719,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
     @mock.patch('nova_powervm.virt.powervm.tasks.vm.UpdateIBMiSettings'
                 '.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_get_boot_connectivity_type')
     @mock.patch('pypowervm.tasks.power.power_on')
     def test_spawn_ibmi_without_bdms(self, mock_pwron, mock_boot_conn_type,
@@ -863,44 +853,40 @@ class TestPowerVMDriver(test.TestCase):
         ret = self.drv._is_booted_from_volume(None)
         self.assertFalse(ret)
 
-    @mock.patch('nova_powervm.virt.powervm.driver.VOLUME_DRIVER_MAPPINGS')
-    def test_get_inst_xag(self, mock_mapping):
-        def getitem(name):
-            return self._vol_drv_maps()[name]
-        mock_mapping.__getitem__.side_effect = getitem
-
-        self.flags(volume_adapter='fibre_channel', group='powervm')
+    def test_get_inst_xag(self):
         # No volumes - should be just the SCSI mapping
         xag = self.drv._get_inst_xag(mock.Mock(), None)
         self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
 
         # The vSCSI Volume attach - only needs the SCSI mapping.
         self.flags(fc_attach_strategy='vscsi', group='powervm')
-        xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()])
+        mock_bdm = {'connection_info':
+                    {'driver_volume_type': 'fibre_channel'}}
+        xag = self.drv._get_inst_xag(mock.Mock(), [mock_bdm])
         self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
 
         # The NPIV volume attach - requires SCSI, Storage and FC Mapping
         self.flags(fc_attach_strategy='npiv', group='powervm')
-        mock_mapping.return_value = self._vol_drv_maps()
-        xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()])
-        self.assertEqual({pvm_const.XAG.VIO_STOR,
-                          pvm_const.XAG.VIO_SMAP,
+        xag = self.drv._get_inst_xag(mock.Mock(), [mock_bdm])
+        self.assertEqual({pvm_const.XAG.VIO_STOR, pvm_const.XAG.VIO_SMAP,
                           pvm_const.XAG.VIO_FMAP}, set(xag))
 
         # The vSCSI Volume attach - Ensure case insensitive.
         self.flags(fc_attach_strategy='VSCSI', group='powervm')
-        mock_mapping.return_value = self._vol_drv_maps()
-        xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()])
+        xag = self.drv._get_inst_xag(mock.Mock(), [mock_bdm])
         self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
 
-        # The iSCSI volume attach - only nees the SCSI mapping.
-        self.flags(volume_adapter='iscsi', group='powervm')
-        mock_mapping.return_value = self._vol_drv_maps()
-        xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()])
-        self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
+        # Validate the other volume types only return SCSI mappings
+        vol_types = ['iscsi', 'gpfs', 'local', 'nfs']
+        for vol_type in vol_types:
+            self.flags(volume_adapter='iscsi', group='powervm')
+            mock_bdm = {'connection_info':
+                        {'driver_volume_type': vol_type}}
+            xag = self.drv._get_inst_xag(mock.Mock(), [mock_bdm])
+            self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
 
         # If a recreate, all should be returned
-        xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()], recreate=True)
+        xag = self.drv._get_inst_xag(mock.Mock(), [mock_bdm], recreate=True)
         self.assertEqual({pvm_const.XAG.VIO_STOR,
                           pvm_const.XAG.VIO_SMAP,
                           pvm_const.XAG.VIO_FMAP}, set(xag))
@@ -948,7 +934,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.update',
                 new=mock.Mock())
     @mock.patch('nova_powervm.virt.powervm.tasks.network.UnplugVifs.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.vm.dlt_lpar')
     @mock.patch('nova_powervm.virt.powervm.vm.power_off')
@@ -1110,8 +1096,9 @@ class TestPowerVMDriver(test.TestCase):
                           [], block_device_info=mock_bdms)
         assert_not_called()
 
+    @mock.patch('pypowervm.tasks.scsi_mapper.remove_maps')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.UnplugVifs.execute')
-    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm_ext.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.vm.dlt_lpar')
     @mock.patch('nova_powervm.virt.powervm.vm.power_off')
@@ -1124,12 +1111,13 @@ class TestPowerVMDriver(test.TestCase):
     def test_destroy_internal_no_nvram_cleanup(
             self, mock_bld_slot_mgr, mock_get_flv, mock_cfg, mock_pvmuuid,
             mock_dlt_vopt, mock_pwroff, mock_dlt, mock_boot_from_vol,
-            mock_unplug_vifs):
+            mock_unplug_vifs, mock_rm_maps):
         """Validates the basic PowerVM destroy, without NVRAM cleanup.
 
         Used to validate the behavior when destroying evacuated instances.
         It should not clean up NVRAM as the instance is still on another host.
         """
+        mock_rm_maps.return_value = []
         # NVRAM Manager
         self.drv.nvram_mgr = mock.Mock()
         self.inst.host = 'other'
@@ -1271,6 +1259,7 @@ class TestPowerVMDriver(test.TestCase):
         # Verify the disconnect volume was not invoked
         self.assertEqual(0, self.vol_drv.disconnect_volume.call_count)
 
+    @mock.patch('pypowervm.tasks.scsi_mapper.remove_maps')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.UnplugVifs.execute')
     @mock.patch('nova_powervm.virt.powervm.vm.dlt_lpar')
     @mock.patch('nova_powervm.virt.powervm.vm.power_off')
@@ -1279,9 +1268,11 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.configdrive.required_by')
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
     def test_destroy_rollback(self, mock_get_flv, mock_cfg, mock_dlt_vopt,
-                              mock_pwroff, mock_dlt, mock_unplug_vifs):
+                              mock_pwroff, mock_dlt, mock_unplug_vifs,
+                              mock_rm_maps):
         """Validates the basic PowerVM destroy rollback mechanism works."""
         # Set up the mocks to the tasks.
+        mock_rm_maps.return_value = []
         mock_get_flv.return_value = self.inst.get_flavor()
         mock_cfg.return_value = True
 
@@ -1531,13 +1522,11 @@ class TestPowerVMDriver(test.TestCase):
         """Validates the log_operations."""
         self.drv._log_operation('fake_op', self.inst)
         entry = (r'Operation: %(op)s. Virtual machine display '
-                 'name: %(display_name)s, name: %(name)s, '
-                 'UUID: %(uuid)s')
-        msg_dict = {'uuid': 'b3c04455-a435-499d-ac81-371d2a2d334f',
-                    'display_name': u'Fake Instance',
+                 'name: %(display_name)s, name: %(name)s')
+        msg_dict = {'display_name': u'Fake Instance',
                     'name': 'instance-00000001',
                     'op': 'fake_op'}
-        mock_log.info.assert_called_with(entry, msg_dict)
+        mock_log.info.assert_called_with(entry, msg_dict, instance=self.inst)
 
     @mock.patch('pypowervm.wrappers.managed_system.System.get',
                 new=mock.Mock(return_value=[mock.Mock()]))
@@ -1688,7 +1677,7 @@ class TestPowerVMDriver(test.TestCase):
     def test_get_host_ip_addr(self):
         self.assertEqual(self.drv.get_host_ip_addr(), '127.0.0.1')
 
-    @mock.patch('nova.virt.powervm.driver.LOG.warning')
+    @mock.patch('nova.virt.powervm_ext.driver.LOG.warning')
     @mock.patch('nova.compute.utils.get_machine_ips')
     def test_get_host_ip_addr_failure(self, mock_ips, mock_log):
         mock_ips.return_value = ['1.1.1.1']
@@ -1970,6 +1959,23 @@ class TestPowerVMDriver(test.TestCase):
             self.drv.adapter, self.inst, self.drv.host_uuid,
             force_immediate=False, timeout=500)
 
+    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver._destroy')
+    def test_confirm_migration_diff_host(self, mock_destroy):
+        mock_mig = mock.Mock(source_compute='host1', dest_compute='host2')
+        self.drv.confirm_migration('context', mock_mig, self.lpm_inst,
+                                   'network_info')
+        mock_destroy.assert_called_once_with(
+            'context', self.lpm_inst, block_device_info=None,
+            destroy_disks=False, shutdown=False)
+
+    @mock.patch('nova_powervm.virt.powervm.vm.rename')
+    def test_confirm_migration_same_host(self, mock_rename):
+        mock_mig = mock.Mock(source_compute='host1', dest_compute='host1')
+        self.drv.confirm_migration('context', mock_mig, self.lpm_inst,
+                                   'network_info')
+        mock_rename.assert_called_once_with(
+            self.drv.adapter, self.lpm_inst, self.lpm_inst.name)
+
     def test_post_live_mig_dest(self):
         self.drv.post_live_migration_at_destination(
             'context', self.lpm_inst, 'network_info')
@@ -2026,8 +2032,8 @@ class TestPowerVMDriver(test.TestCase):
 
         def _get_results(block_device_info=None, bdms=None):
             # Patch so we get the same mock back each time.
-            with mock.patch.object(self.drv, '_get_inst_vol_adpt',
-                                   return_value=vol_adpt):
+            with mock.patch('nova_powervm.virt.powervm.volume.'
+                            'build_volume_driver', return_value=vol_adpt):
                 return [
                     (bdm, vol_drv) for bdm, vol_drv in self.drv._vol_drv_iter(
                         'context', self.inst,
